/*****************************************************************************
  BASS_FX 2.2 - Copyright (c) 2002-2005 (: JOBnik! :) [Arthur Aminov, ISRAEL]
                                                      [http://www.jobnik.org]

      bugs/suggestions/questions -> e-mail: bass_fx@jobnik.org
     ----------------------------------------------------------

  NOTE: This header will only work with BASS_FX version 2.2
        Check http://www.un4seen.com for any later versions of BASS_FX.H

  * Requires BASS version 2.2 - available @ www.un4seen.com 
*****************************************************************************/

#ifndef BASS_FX_H
#define BASS_FX_H

#ifdef __cplusplus
  extern "C" {
#endif

#ifndef BASS_FXDEF
  #define BASS_FXDEF(f) WINAPI f
#endif

// Error codes returned by BASS_ErrorGetCode()
#define BASS_FX_ERROR_NODECODE	100		// Not a decoding channel
#define BASS_FX_ERROR_STEREO	101		// Only for stereo
#define BASS_FX_ERROR_BPMINUSE	102		// BPM detection is in use
#define BASS_FX_ERROR_BPMX2		103		// BPM has already multed by 2

// Tempo / Reverse / BPM flag
#define BASS_FX_FREESOURCE		0x10000	// Free the source handle as well?

/*=============================================================================================
	D S P (Digital Signal Processing)
==============================================================================================*/

// DSP effects
enum{
	BASS_FX_DSPFX_SWAP,			// Swap channels: left<=>right : for STEREO Only!
	BASS_FX_DSPFX_ROTATE,		// A channels volume ping-pong : for STEREO Only!
	BASS_FX_DSPFX_ECHO,			// Echo
	BASS_FX_DSPFX_FLANGER,		// Flanger
	BASS_FX_DSPFX_VOLUME,		// Volume Amplifier : L/R for STEREO, L for MONO
	BASS_FX_DSPFX_PEAKEQ,		// Peaking Equalizer
	BASS_FX_DSPFX_REVERB,		// Reverb
	BASS_FX_DSPFX_LPF,			// Low Pass Filter
	BASS_FX_DSPFX_S2M,			// Stereo 2 Mono : for STEREO Only!
	BASS_FX_DSPFX_DAMP,			// Dynamic Amplification
	BASS_FX_DSPFX_AUTOWAH,		// Auto WAH
	BASS_FX_DSPFX_ECHO2,		// Echo 2
	BASS_FX_DSPFX_PHASER,		// Phaser
	BASS_FX_DSPFX_ECHO3,		// Echo 3
	BASS_FX_DSPFX_CHORUS,		// Chorus
	BASS_FX_DSPFX_APF,			// All Pass Filter
	BASS_FX_DSPFX_COMPRESSOR,	// Compressor
	BASS_FX_DSPFX_DISTORTION	// Distortion
};

// Flanger
typedef struct{
	float fWetDry;				// [0.........1]
	float fSpeed;				// [0.........n]
} BASS_FX_DSPFLANGER;

// Echo
typedef struct{
	float fLevel;				// [0.........1]
	int   lDelay;				// [1200..30000]
} BASS_FX_DSPECHO;

// Reverb
typedef struct{
	float fLevel;				// [0.........1]
	int   lDelay;				// [1200..10000]
} BASS_FX_DSPREVERB;

// Volume
typedef struct{					// L/R for STEREO, L for MONO for both channels
	float fLeft;				// [0....1....n]
	float fRight;				// [0....1....n]
} BASS_FX_DSPVOLUME;

// Peaking Equalizer
typedef struct{
	int   lBand;				// [0..n] more bands => more memory & cpu usage
	int   lFreq;				// current samplerate of a stream/music
	float fBandwidth;			// in octaves [0..4] - Q is not in use
	float fQ;					// the EE kinda definition	[0..1] - Bandwidth is not in use
	float fCenter;				// eg. 1000Hz
	float fGain;				// in dB. [-15..0..+15]
} BASS_FX_DSPPEAKEQ;

// Low Pass Filter
typedef struct{
	int   lFreq;				// current samplerate of stream/music
	float fResonance;			// [2..........10]
	float fCutOffFreq;			// [200Hz..5000Hz]
} BASS_FX_DSPLPF;

// Dynamic Amplification
typedef struct {
	float fTarget;				// target volume level						[0.......1]
	float fQuiet; 				// quiet  volume level						[0.......1]
	float fRate;				// amp adjustment rate						[e.g: 0.02]
	float fGain;				// amplification level						[0.......1]
	float fDelay;				// delay in seconds before increasing level	[0..Length]
} BASS_FX_DSPDAMP;

// Auto WAH
typedef struct {
	float fDryMix;				// dry (unaffected) signal mix				[-2......2]
	float fWetMix;				// wet (affected) signal mix				[-2......2]
	float fFeedback;			// feedback									[-1......1]
	float fRate;				// rate of sweep in cycles per second		[0<....<10]
	float fRange;				// sweep range in octaves					[0<....<10]
	float fFreq;				// base frequency of sweep Hz				[0<...1000]
} BASS_FX_DSPAUTOWAH;

// Echo 2
typedef struct {
	float fDryMix;				// dry (unaffected) signal mix				[-2......2]
	float fWetMix;				// wet (affected) signal mix				[-2......2]
	float fFeedback;			// feedback									[-1......1]
	float fDelay;				// delay sec								[0<......6]
} BASS_FX_DSPECHO2;

// Phaser
typedef struct {
	float fDryMix;				// dry (unaffected) signal mix				[-2......2]
	float fWetMix;				// wet (affected) signal mix				[-2......2]
	float fFeedback;			// feedback									[-1......1]
	float fRate;				// rate of sweep in cycles per second		[0<....<10]
	float fRange;				// sweep range in octaves					[0<....<10]
	float fFreq;				// base frequency of sweep					[0<...1000]
} BASS_FX_DSPPHASER;

// Echo 3
typedef struct {
	float fDryMix;				// dry (unaffected) signal mix				[-2......2]
	float fWetMix;				// wet (affected) signal mix				[-2......2]
	float fDelay;				// delay sec								[0<......6]
} BASS_FX_DSPECHO3;

// Chorus
typedef struct {
	float fDryMix;				// dry (unaffected) signal mix				[-2......2]
	float fWetMix;				// wet (affected) signal mix				[-2......2]
	float fFeedback;			// feedback									[-1......1]
	float fMinSweep;			// minimal delay ms							[0<...6000]
	float fMaxSweep;			// maximum delay ms							[0<...6000]
	float fRate;				// rate ms/s								[0<...1000]
} BASS_FX_DSPCHORUS;

// All Pass Filter
typedef struct {
	float fGain;				// reverberation time						[-1=<..<=1]
	float fDelay;				// delay sec								[0<....<=6]
} BASS_FX_DSPAPF;

// Compressor
typedef struct {
	float fThreshold;			// compressor threshold						[0<=...<=1]
	float fAttacktime;			// attack time ms							[0<.<=1000]
	float fReleasetime;			// release time ms							[0<.<=5000]
} BASS_FX_DSPCOMPRESSOR;

// Distortion
typedef struct {
	float fDrive;				// distortion drive							[0<=...<=5]
	float fDryMix;				// dry (unaffected) signal mix				[-5<=..<=5]
	float fWetMix;				// wet (affected) signal mix				[-5<=..<=5]
	float fFeedback;			// feedback									[-1<=..<=1]
	float fVolume;				// distortion volume						[0=<...<=2]
} BASS_FX_DSPDISTORTION;


BOOL BASS_FXDEF(BASS_FX_DSP_Set)(DWORD handle, int dsp_fx, int priority);
/*
	Set any chosen DSP effect to any handle.
	handle   : stream/music/wma/cd/any other supported add-on format
	dsp_fx   : FX you wish to use
	priority : The priority of the new DSP, which determines it's position in the DSP chain
			   DSPs with higher priority are called before those with lower.
	RETURN   : TRUE if created (0=error!)
*/

BOOL BASS_FXDEF(BASS_FX_DSP_Remove)(DWORD handle, int dsp_fx);
/*
	Remove chosen DSP effect.
	handle : stream/music/wma/cd/any other supported add-on format
	dsp_fx : FX you wish to remove
	RETURN : TRUE if removed (0=error!)
*/

BOOL BASS_FXDEF(BASS_FX_DSP_SetParameters)(DWORD handle, int dsp_fx, void *par);
/* 
	Set the parameters of a DSP effect.
	handle : stream/music/wma/cd/any other supported add-on format
	dsp_fx : FX you wish to set parameters to
	par    : Pointer to the parameter structure
	RETURN : TRUE if ok (0=error!)
*/

BOOL BASS_FXDEF(BASS_FX_DSP_GetParameters)(DWORD handle, int dsp_fx, void *par);
/* 
	Retrieve the parameters of a DSP effect.
	handle : stream/music/wma/cd/any other supported add-on format
	dsp_fx : FX you wish to get parameters from
	par    : Pointer to the parameter structure
	RETURN : TRUE if ok (0=error!)
*/

BOOL BASS_FXDEF(BASS_FX_DSP_Reset)(DWORD handle, int dsp_fx);
/*
	Call this function before changing position to avoid *clicks*
	handle : stream/music/wma/cd/any other supported add-on format
	dsp_fx : FX you wish to reset parameters of
	RETURN : TRUE if ok (0=error!)
*/

/*=============================================================================================
	TEMPO / PITCH SCALING / SAMPLERATE
==============================================================================================*/

// tempo flags
#define BASS_FX_TEMPO_QUICKALGO		0x400000 // Using quicker tempo change algorithm gain speed, lose quality
#define BASS_FX_TEMPO_NO_AAFILTER	0x800000 // Not using FIR low-pass (anti-alias) filter gain speed, lose quality

// NOTE: Enable Tempo supported flags in BASS_FX_TempoCreate and the others to source handle.

HSTREAM BASS_FXDEF(BASS_FX_TempoCreate)(DWORD chan, DWORD flags);
/*
	Creates a resampling stream from a decoding channel.
	chan     : a handle returned by:
					BASS_StreamCreateFile     : flags = BASS_STREAM_DECODE ...
					BASS_MusicLoad            : flags = BASS_MUSIC_DECODE ...
					BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...
					BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...
				    * Any other add-on handle using a decoding channel.
	flags    : BASS_SAMPLE_SOFTWARE/LOOP/3D/FX or BASS_STREAM_DECODE/AUTOFREE or
	           BASS_SPEAKER_xxx or BASS_FX_TEMPO_xxx or BASS_FX_FREESOURCE
	RETURN   : the tempo stream handle (0=error!)
*/

DWORD BASS_FXDEF(BASS_FX_TempoGetSource)(HSTREAM chan);
/*
	Get the source channel handle.
	chan   : tempo stream handle
	RETURN : the source channel handle (0=error!)
*/

BOOL  BASS_FXDEF(BASS_FX_TempoSet)(HSTREAM chan, float tempo, float samplerate, float pitch);
/*
	Set new values to tempo/rate/pitch to change its speed.
	chan       : tempo stream (or source channel) handle
	tempo      : in Percents  [-95%..0..+5000%]					(-100 = leave current)
	samplerate : in Hz, but calculates by the same % as tempo	(   0 = leave current)
	pitch      : in Semitones [-60....0....+60]					(-100 = leave current)
	RETURN     : TRUE if ok (0=error!)
*/

BOOL  BASS_FXDEF(BASS_FX_TempoGet)(HSTREAM chan, float *tempo, float *samplerate, float *pitch);
/*
	Get tempo/rate/pitch values.
	chan       : tempo stream (or source channel) handle
	tempo      : current tempo			(NULL if not in use)
	samplerate : current samplerate		(NULL if not in use)
	pitch      : current pitch			(NULL if not in use)
	RETURN     : TRUE if ok (0=error!)
*/

float BASS_FXDEF(BASS_FX_TempoGetRateRatio)(HSTREAM chan);
/*
	Get the ratio of the resulting rate and source rate (the resampling ratio).
	chan   : tempo stream (or source channel) handle
	RETURN : the ratio (0=error!)
*/

/*=============================================================================================
	R E V E R S E
==============================================================================================*/

// NOTES: 1. MODs won't load without BASS_MUSIC_PRESCAN flag.
//		  2. Enable Reverse supported flags in BASS_FX_ReverseCreate and the others to source handle.

HSTREAM BASS_FXDEF(BASS_FX_ReverseCreate)(HSTREAM chan, float dec_block, DWORD flags);
/*
	Creates a Reversed stream from a decoding channel.
	chan      : a handle returned by:
					BASS_StreamCreateFile	  : flags = BASS_STREAM_DECODE ...
					BASS_MusicLoad            : flags = BASS_MUSIC_DECODE | BASS_MUSIC_PRESCAN ...
					BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...
					BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...
					* Other stream add-on formats if created as decoded channel.
				* For better MP3/2/1 Reverse playback use: BASS_STREAM_PRESCAN flag
	dec_block : decode in # of seconds blocks...
				larger blocks = less seeking overhead but larger spikes.
	flags	  : BASS_SAMPLE_SOFTWARE/LOOP/3D/FX or BASS_STREAM_DECODE/AUTOFREE or
				BASS_SPEAKER_xxx or BASS_FX_FREESOURCE
	RETURN    : the reverse stream handle (0=error!)
*/

HSTREAM BASS_FXDEF(BASS_FX_ReverseGetSource)(HSTREAM chan);
/*
	Get the source channel handle.
	chan   : reverse stream handle
	RETURN : the source channel handle (0=error!)
*/

/*=============================================================================================
	B P M (Beats Per Minute)
==============================================================================================*/

// bpm flags
#define BASS_FX_BPM_BKGRND	1	// If in use, then you can do other stuff while detection's in process
					// not available in MacOSX yet.
#define BASS_FX_BPM_MULT2	2	// If in use, then will auto multiply bpm by 2 (if BPM < MinBPM*2)

//-----------
// Option - 1 - Get BPM from a decoded channel
//--------------------------------------------
typedef void (CALLBACK BPMPROCESSPROC)(DWORD chan, float percent);
/*
	Get the detection process in percents of a channel.
	chan    : channel that the BASS_FX_BPM_DecodeGet has applied to
	percent : the process in percents [0%..100%]
*/

float BASS_FXDEF(BASS_FX_BPM_DecodeGet)(DWORD chan, float startSec, float endSec, DWORD minMaxBPM, DWORD flags, BPMPROCESSPROC *proc);
/*
	Get the original BPM of a decoding channel.
	chan	  : a handle returned by:
					BASS_StreamCreateFile     : flags = BASS_STREAM_DECODE ...
					BASS_MusicLoad            : flags = BASS_MUSIC_DECODE | BASS_MUSIC_PRESCAN ...
					BASS_WMA_StreamCreateFile : flags = BASS_STREAM_DECODE ...
					BASS_CD_StreamCreate      : flags = BASS_STREAM_DECODE ...
					* Any other add-on handle using a decoding channel.
	startSec  : start detecting position in seconds
	endSec    : end detecting position in seconds
	minMaxBPM : set min & max bpm, e.g: MAKELONG(LOWORD.HIWORD), LO=Min, HI=Max. 0 = defaults 45/230
	flags	  : BASS_FX_BPM_xxx or BASS_FX_FREESOURCE
	*proc     : user defined function to receive the process in percents, use NULL if not in use
	RETURN	  : the original BPM value (-1=error!)
*/

//-----------
// Option - 2 - Auto get BPM by period of time in seconds
//-------------------------------------------------------
typedef void (CALLBACK BPMPROC)(DWORD handle, float bpm, DWORD user);
/*
	Get the BPM after period of time in seconds.
	handle : handle that the BASS_FX_BPM_CallbackSet has applied to
	bpm    : the new original bpm value
	user   : the user instance data given when BASS_FX_BPM_CallbackSet was called.
*/

BOOL BASS_FXDEF(BASS_FX_BPM_CallbackSet)(DWORD handle, BPMPROC *proc, float period, DWORD minMaxBPM, DWORD flags, DWORD user);
/*
	Enable getting BPM value by period of time in seconds.
	handle    : stream/music/wma/cd/any other supported add-on format
	*proc     : user defined function to receive the bpm value
	period	  : detection period in seconds
	minMaxBPM : set min & max bpm, e.g: MAKELONG(LOWORD.HIWORD), LO=Min, HI=Max. 0 = defaults 45/230
	flags     : only BASS_FX_BPM_MULT2 flag is used
	user      : user instance data to pass to the callback function.
	RETURN    : TRUE if ok (0=error!)
*/

BOOL BASS_FXDEF(BASS_FX_BPM_CallbackReset)(DWORD handle);
/*
	Reset the buffers. Call this function after changing position.
	handle : stream/music/wma/cd/any other supported add-on format
	RETURN : TRUE if ok (0=error!)
*/

/*---------------------------------------
   Functions to use with Both options.
---------------------------------------*/

// translation options
#define BASS_FX_BPM_X2			0	// Multiply the original BPM value by 2 (may be called only once & will change the original BPM as well!).
#define BASS_FX_BPM_2FREQ		1	// BPM value to Frequency.
#define BASS_FX_BPM_FREQ2		2	// Frequency to BPM value.
#define BASS_FX_BPM_2PERCENT	3	// Percents to BPM value.
#define BASS_FX_BPM_PERCENT2	4	// BPM value to Percents.

float BASS_FXDEF(BASS_FX_BPM_Translate)(DWORD handle, float val2tran, DWORD trans);
/*
	Translate the given BPM to FREQ/PERCENT and vice versa or multiply BPM by 2.
	handle   : stream/music/wma/cd/any other supported add-on format
	val2tran : specify a value to translate to a given option (no matter if used X2).
	trans    : any of the above translation option
	RETURN   : new calculated value. (-1=error!)	

	NOTE     : This function will not detect the BPM, it will just translate the detected 
		       original BPM value of a given handle.
*/

BOOL  BASS_FXDEF(BASS_FX_BPM_Free)(DWORD handle);
/*
	Free all resources used by a given handle.

	NOTE: If BASS_FX_FREESOURCE is used, then will free the decoding channel as well.
          You can't set/get this flag with BASS_ChannelSetFlags/BASS_ChannelGetInfo.

	handle : stream/music/wma/cd/any other supported add-on format
*/

#ifdef __cplusplus
}
#endif

#endif