/***************************************************************************
 dsp.c/h/rc - Copyright (c) 2002-2005 (: JOBnik! :) [Arthur Aminov, ISRAEL]
                                                    [http://www.jobnik.org]
                                                    [   bass_fx@jobnik.org]
 
 BASS_FX dsp fx in action
 * Imports: bass.lib, bass_fx.lib
            kernel32.lib, user32.lib, comdlg32.lib, gdi32.lib
***************************************************************************/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "bass.h"
#include "bass_fx.h"
#include "dsp.h"

HWND win=NULL;
HINSTANCE inst;

HFONT Font;					// font handle
DWORD chan;					// a handle
DWORD floatable;			// floating-point channel support?

#define maxp 1.3			// increase dx rate for max 30%
#define minp 0.7			// decrease dx rate for min 30%

#define MESS(id,m,w,l) SendDlgItemMessage(win,id,m,(WPARAM)w,(LPARAM)l)

OPENFILENAME ofn;
char path[MAX_PATH];

// set v/h scrollers
DWORD SetScroller(LPARAM l, WPARAM w, int _LINE, int _PAGE, DWORD _MAX)
{
	DWORD a = SendMessage((HWND)l,SBM_GETPOS,0,0);

	switch (LOWORD(w)) {
		case SB_THUMBTRACK:
		case SB_THUMBPOSITION:
			a = HIWORD(w);
		break;
		case SB_LINELEFT:
			if(a == 0)return a;
			a = max( a - _LINE, 0);
		break;
		case SB_LINERIGHT:
			a = (int)min(a + _LINE, _MAX);
		break;
		case SB_PAGELEFT:
			a = max(a - _PAGE, 0);
		break;
		case SB_PAGERIGHT:
			a = (int)min(a + _PAGE, _MAX);
		break;
	}
	return a;
}

// display error messages
void Error(char *es)
{
	char mes[200];
	sprintf(mes,"%s\n\nBASS error code: %d",es,BASS_ErrorGetCode());
	MessageBox(win,mes,"Error",MB_ICONEXCLAMATION);
}

// Update DSP EQ
void UpdateFX(int b)
{
	BASS_FX_DSPPEAKEQ eq;		// dsp peaking equalizer
	int v = MESS(ID_C0+b,SBM_GETPOS,0,0);

	eq.lBand = b;	// get values of the selected band
	BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
		eq.fGain = (float)(10 - v);
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
}

// Set DSP EQ
void SetDSP_EQ(float fGain, float fBandwidth, float fQ, float fCenter_Bass, float fCenter_Mid, float fCenter_Treble)
{
	BASS_FX_DSPPEAKEQ eq;		// dsp peaking equalizer
	DWORD f;

	// Get current sample rate
	BASS_ChannelGetAttributes(chan, &f, NULL, NULL);

	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 1st band
	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 2nd band
	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 3rd band...

	eq.lFreq = f;
	eq.fGain = fGain;
	eq.fQ = fQ;
	eq.fBandwidth = fBandwidth;

	// set bass
	eq.lBand = 0;
	eq.fCenter = fCenter_Bass;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// set mid
	eq.lBand = 1;
	eq.fCenter = fCenter_Mid;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// set treble
	eq.lBand = 2;
	eq.fCenter = fCenter_Treble;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// update dsp eq
	UpdateFX(0);
	UpdateFX(1);
	UpdateFX(2);
}

// get the file name from the file path
char *GetFileName(char *filepath)
{
	unsigned char slash_location;
	filepath = strrev(filepath);
	slash_location = strchr(filepath, '\\') - filepath;
	return (strrev(filepath) + strlen(filepath) - slash_location);
}

BOOL CALLBACK dialogproc(HWND h,UINT m,WPARAM w,LPARAM l)
{
	DWORD a = 0, freq;
	int MAX = 0, LINE = 0, PAGE = 0;
	char c[30];

	BASS_FX_DSPPEAKEQ eq;		// dsp peaking equalizer
	BASS_FX_DSPPHASER phs;		// dsp phaser

	switch (m) {
		case WM_COMMAND:
			switch (LOWORD(w)) {
				case ID_OPEN:
					{
						char file[MAX_PATH]="";
						ofn.lpstrFilter="playable files\0*.mo3;*.xm;*.mod;*.s3m;*.it;*.mtm;*.mp3;*.mp2;*.mp1;*.ogg;*.wav;*.aif\0All files\0*.*\0\0";
						ofn.lpstrFile=file;
						if (GetOpenFileName(&ofn)) {
							memcpy(path,file,ofn.nFileOffset);
							path[ofn.nFileOffset-1]=0;

							// free previous dsp effects & handles
							BASS_StreamFree(chan);	// free stream
							BASS_MusicFree(chan);	// free music

							if(!(chan = BASS_StreamCreateFile(FALSE, file, 0, 0, BASS_SAMPLE_LOOP|floatable))&&
								!(chan = BASS_MusicLoad(FALSE, file, 0, 0, BASS_MUSIC_LOOP|BASS_MUSIC_RAMP|floatable,0))){
								// not a WAV/MP3 or MOD
								MESS(ID_OPEN,WM_SETTEXT,0,"click here to open a file && play it...");
								Error("Selected file couldnt be loaded!");
								break;
							}

							// update the Button to show the loaded file name
							MESS(ID_OPEN,WM_SETTEXT,0,GetFileName(file));

							// set dsp effects
							SendMessage(win,WM_COMMAND,IDC_CHKEQ,l);
							SendMessage(win,WM_COMMAND,IDC_CHKPHS,l);

							// get current sample rate
							BASS_ChannelGetAttributes(chan, &freq, NULL, NULL);

							// set the dx sample rate & view
							MESS(IDC_DXRATE,TBM_SETRANGEMAX,0,(long)((float)freq * maxp));
							MESS(IDC_DXRATE,TBM_SETRANGEMIN,0,(long)((float)freq * minp));
							MESS(IDC_DXRATE,TBM_SETPOS,TRUE,freq);
							MESS(IDC_DXRATE,TBM_SETPAGESIZE,0,(long)((float)freq * 0.01f));	// by 1%

							sprintf(c,"DirectX Samplerate = %dHz", freq);
							MESS(IDC_SDXRATE,WM_SETTEXT,0,c);

							// it must be one of them :)
							BASS_ChannelPlay(chan, FALSE);
						}
					}
					return 1;

				case IDC_CHKEQ:
					if (MESS(IDC_CHKEQ,BM_GETCHECK,0,0))
						SetDSP_EQ(0.0f, 2.5f, 0.0f, 125.0f, 1000.0f, 8000.0f);
					else
						BASS_FX_DSP_Remove(chan, BASS_FX_DSPFX_PEAKEQ);
				return 1;

				case IDC_CHKPHS:
					if(MESS(IDC_CHKPHS,BM_GETCHECK,0,0)){
						BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PHASER,0);

						BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PHASER, &phs);
							phs.fWetMix = (float)MESS(IDC_WETMIX,TBM_GETPOS,0,0) / 1000.0f;
							phs.fDryMix = (float)MESS(IDC_DRYMIX,TBM_GETPOS,0,0) / 1000.0f;
							phs.fFeedback = (float)MESS(IDC_FEEDBACK,TBM_GETPOS,0,0) / 1000.0f;
							phs.fRate = (float)MESS(IDC_RATE,TBM_GETPOS,0,0) / 10.0f;
							phs.fRange = (float)MESS(IDC_RANGE,TBM_GETPOS,0,0) / 10.0f;
							phs.fFreq = (float)MESS(IDC_FREQ,TBM_GETPOS,0,0) / 10.0f;
						BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PHASER, &phs);
					}else
						BASS_FX_DSP_Remove(chan, BASS_FX_DSPFX_PHASER);
				return 1;

			}
			return 1;

		case WM_VSCROLL:
			PAGE = 5;
			LINE = 1;
			MAX  = 20;
			a = SetScroller(l,w,LINE,PAGE,MAX);
			SendMessage((HWND)l,SBM_SETPOS,a,1);
			UpdateFX(GetDlgCtrlID((HWND)l)-ID_C0);
		return 1;

		case WM_HSCROLL:
			if(!BASS_ChannelIsActive(chan)) break;

			switch (GetDlgCtrlID((HWND)l)) {
				case IDC_DXRATE:
					BASS_ChannelSetAttributes(chan, MESS(IDC_DXRATE, TBM_GETPOS, 0, 0), -1, -101);

					sprintf(c,"DirectX Samplerate = %dHz", MESS(IDC_DXRATE, TBM_GETPOS, 0, 0));
					MESS(IDC_SDXRATE,WM_SETTEXT,0,c);

					// Update eq sample rate after changing sample rate
					eq.lBand = 0;	// we must get some band info :)
					BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
						eq.lFreq = MESS(IDC_DXRATE, TBM_GETPOS, 0, 0);
					BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
				break;

				case IDC_DRYMIX:
				case IDC_WETMIX:
				case IDC_FEEDBACK:
				case IDC_RATE:
				case IDC_RANGE:
				case IDC_FREQ:
					BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PHASER, &phs);
						phs.fWetMix = (float)MESS(IDC_WETMIX,TBM_GETPOS,0,0) / 1000.0f;
						phs.fDryMix = (float)MESS(IDC_DRYMIX,TBM_GETPOS,0,0) / 1000.0f;
						phs.fFeedback = (float)MESS(IDC_FEEDBACK,TBM_GETPOS,0,0) / 1000.0f;
						phs.fRate = (float)MESS(IDC_RATE,TBM_GETPOS,0,0) / 10.0f;
						phs.fRange = (float)MESS(IDC_RANGE,TBM_GETPOS,0,0) / 10.0f;
						phs.fFreq = (float)MESS(IDC_FREQ,TBM_GETPOS,0,0) / 10.0f;
					BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PHASER, &phs);
			}
			return 1;

		case WM_INITDIALOG:
			win=h;
			GetCurrentDirectory(MAX_PATH,path);
			memset(&ofn,0,sizeof(ofn));
			ofn.lStructSize=sizeof(ofn);
			ofn.hwndOwner=h;
			ofn.hInstance=inst;
			ofn.nMaxFile=MAX_PATH;
			ofn.Flags=OFN_HIDEREADONLY|OFN_EXPLORER;

			// enable floating-point DSP
			BASS_SetConfig(BASS_CONFIG_FLOATDSP, TRUE);

			// setup output - default device, 44100hz, stereo, 16 bits
			if (!BASS_Init(-1,44100,0,win,NULL)) {
				Error("Can't initialize device");
				DestroyWindow(win);
				return 1;
			}

			// check for floating-point capability
			floatable = BASS_StreamCreate(44100, 2, BASS_SAMPLE_FLOAT, 0, 0);
			if (floatable) {
				BASS_StreamFree(floatable);  //woohoo!
			    floatable = BASS_SAMPLE_FLOAT;
			}

			// initialize dsp eq sliders
			MESS(ID_C0,SBM_SETRANGE,0,20);
			MESS(ID_C0,SBM_SETPOS,4,1);

			MESS(ID_C1,SBM_SETRANGE,0,20);
			MESS(ID_C1,SBM_SETPOS,10,1);

			MESS(ID_C2,SBM_SETRANGE,0,20);
			MESS(ID_C2,SBM_SETPOS,0,1);
    
			// dx rate
			MESS(IDC_DXRATE,TBM_SETRANGEMAX,0,(long)((float)44100*maxp));
			MESS(IDC_DXRATE,TBM_SETRANGEMIN,0,(long)((float)44100*minp));
			MESS(IDC_DXRATE,TBM_SETPOS,TRUE,44100);
			MESS(IDC_DXRATE,TBM_SETPAGESIZE,0,(long)((float)44100*0.01f));	// by 1%

			// DryMix
			MESS(IDC_DRYMIX,TBM_SETRANGEMAX,0,2000);
			MESS(IDC_DRYMIX,TBM_SETRANGEMIN,0,-2000);
			MESS(IDC_DRYMIX,TBM_SETPOS,TRUE,-999);

			// WetMix
			MESS(IDC_WETMIX,TBM_SETRANGEMAX,0,2000);
			MESS(IDC_WETMIX,TBM_SETRANGEMIN,0,-2000);
			MESS(IDC_WETMIX,TBM_SETPOS,TRUE,999);

			// Feedback
			MESS(IDC_FEEDBACK,TBM_SETRANGEMAX,0,1000);
			MESS(IDC_FEEDBACK,TBM_SETRANGEMIN,0,-1000);
			MESS(IDC_FEEDBACK,TBM_SETPOS,TRUE,-60);

			// Rate
			MESS(IDC_RATE,TBM_SETRANGEMAX,0,100);
			MESS(IDC_RATE,TBM_SETRANGEMIN,0,0);
			MESS(IDC_RATE,TBM_SETPOS,TRUE,2);

			// Range
			MESS(IDC_RANGE,TBM_SETRANGEMAX,0,100);
			MESS(IDC_RANGE,TBM_SETRANGEMIN,0,0);
			MESS(IDC_RANGE,TBM_SETPOS,TRUE,60);

			// Freq
			MESS(IDC_FREQ,TBM_SETRANGEMAX,0,10000);
			MESS(IDC_FREQ,TBM_SETRANGEMIN,0,0);
			MESS(IDC_FREQ,TBM_SETPOS,TRUE,1000);

			Font = CreateFont(-13,0,0,0,FW_BOLD,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_STRING_PRECIS,
				              CLIP_STROKE_PRECIS,DRAFT_QUALITY,DEFAULT_PITCH | FF_DONTCARE,
							  "MS Sans Serif");
			// set the font for check boxes
			MESS(IDC_CHKEQ, WM_SETFONT, Font, TRUE);
			MESS(IDC_CHKPHS,WM_SETFONT, Font, TRUE);
		return 1;

		case WM_CLOSE:
			EndDialog(h,0);
			return 0;
		break;
	}
	return 0;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
	inst=hInstance;

	// check that BASS 2.2 was loaded
	if (BASS_GetVersion()!=MAKELONG(2,2)) {
		MessageBox(0,"BASS version 2.2 was not loaded!","Incorrect BASS.DLL",MB_ICONERROR);
		return 1;
	}

	DialogBox(inst,(char*)1000,0,&dialogproc);

	BASS_Free();

	DeleteObject(Font);

	return 0;
}