/*****************************************************************************
 tempo.c/h/rc - Copyright (c) 2003-2005 (: JOBnik! :) [Arthur Aminov, ISRAEL]
                                                      [http://www.jobnik.org]
                                                      [   bass_fx@jobnik.org]
 
 BASS_FX tempo / rate / pitch with dsp fx
 * Imports: bass.lib, bass_fx.lib
            kernel32.lib, user32.lib, comdlg32.lib, gdi32.lib
*****************************************************************************/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "bass.h"
#include "bass_fx.h"
#include "tempo.h"

HWND win=NULL;
HINSTANCE inst;

HSTREAM chan;			// tempo channel handle
HFONT Font;				// font handle

#define maxp	1.3		// increase rate for max 30%
#define minp	0.7		// decrease rate for min 30%

#define MESS(id,m,w,l) SendDlgItemMessage(win,id,m,(WPARAM)w,(LPARAM)l)

OPENFILENAME ofn;
char path[MAX_PATH];

// set v/h scrollers
DWORD SetScroller(LPARAM l, WPARAM w, int _LINE, int _PAGE, DWORD _MAX)
{
	DWORD a = SendMessage((HWND)l,SBM_GETPOS,0,0);

	switch (LOWORD(w)) {
		case SB_THUMBTRACK:
		case SB_THUMBPOSITION:
			a = HIWORD(w);
		break;
		case SB_LINELEFT:
			if(a == 0)return a;
			a = max( a - _LINE, 0);
		break;
		case SB_LINERIGHT:
			a = (int)min(a + _LINE, _MAX);
		break;
		case SB_PAGELEFT:
			a = max(a - _PAGE, 0);
		break;
		case SB_PAGERIGHT:
			a = (int)min(a + _PAGE, _MAX);
		break;
	}
	return a;
}

// display error messages
void Error(char *es)
{
	char mes[200];
	sprintf(mes,"%s\n\nBASS error code: %d",es,BASS_ErrorGetCode());
	MessageBox(win,mes,"Error",MB_ICONEXCLAMATION);
}

// Update DSP EQ
void UpdateFX(int b)
{
	BASS_FX_DSPPEAKEQ eq;	// DSP Peaking Equalizer

	int v = MESS(ID_C0+b,SBM_GETPOS,0,0);
    
	eq.lBand = b;	// get values of the selected band
    BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
		eq.fGain = (float)(10 - v);
    BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
}

// Set DSP EQ
void SetDSP_EQ(float fGain, float fBandwidth, float fQ, float fCenter_Bass, float fCenter_Mid, float fCenter_Treble)
{
	BASS_FX_DSPPEAKEQ eq;	// DSP Peaking Equalizer

	DWORD f;

	// Get current sample rate
	BASS_ChannelGetAttributes(chan, &f, NULL, NULL);

	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 1st band
	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 2nd band
	BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ,0);	// 3rd band...

	eq.lFreq = f;
	eq.fGain = fGain;
	eq.fQ = fQ;
	eq.fBandwidth = fBandwidth;

	// set bass
	eq.lBand = 0;
	eq.fCenter = fCenter_Bass;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// set mid
	eq.lBand = 1;
	eq.fCenter = fCenter_Mid;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// set treble
	eq.lBand = 2;
	eq.fCenter = fCenter_Treble;
	BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);

	// update dsp eq
	UpdateFX(0);
	UpdateFX(1);
	UpdateFX(2);
}

// show the approximate position in MM:SS format
void UpdatePositionLabel(void)
{
	if(!BASS_FX_TempoGetRateRatio(chan)) return;
	{
		char c[30];
		float totalsec=(float)MESS(IDC_POS,TBM_GETRANGEMAX,0,0)/BASS_FX_TempoGetRateRatio(chan);
		float posec=(float)MESS(IDC_POS,TBM_GETPOS,0,0)/BASS_FX_TempoGetRateRatio(chan);
		sprintf(c,"Playing position: %02d:%02d / %02d:%02d", (int)posec/60,(int)posec%60,(int)totalsec/60,(int)totalsec%60);
		MESS(IDC_SPOS,WM_SETTEXT,0,c);
	}
}

// get the file name from the file path
char *GetFileName(char *filepath)
{
	unsigned char slash_location;
	filepath = strrev(filepath);
	slash_location = strchr(filepath, '\\') - filepath;
	return (strrev(filepath) + strlen(filepath) - slash_location);
}

BOOL CALLBACK dialogproc(HWND h,UINT m,WPARAM w,LPARAM l)
{
	DWORD MAX = 20,PAGE = 5,LINE = 1, p, freq;
	char c[30];

	switch (m) {
		case WM_COMMAND:
			switch (LOWORD(w)) {
				case ID_OPEN:
					{
						char file[MAX_PATH]="";
						ofn.lpstrFilter="playable files\0*.mo3;*.xm;*.mod;*.s3m;*.it;*.mtm;*.mp3;*.mp2;*.mp1;*.ogg;*.wav;*aif\0All files\0*.*\0\0";
						ofn.lpstrFile=file;
						if (GetOpenFileName(&ofn)) {
							memcpy(path,file,ofn.nFileOffset);
							path[ofn.nFileOffset-1]=0;

							// free previous streams/dsp handles
							BASS_StreamFree(chan);

							// create decode channel
							chan = BASS_StreamCreateFile(FALSE, file, 0, 0, BASS_STREAM_DECODE);

							// check for MOD
							if (!chan) chan = BASS_MusicLoad(FALSE, file, 0, 0, BASS_MUSIC_RAMP|BASS_MUSIC_PRESCAN|BASS_STREAM_DECODE,0);

							if (!chan){
								// not a WAV/MP3 or MOD
								MESS(ID_OPEN,WM_SETTEXT,0,"click here to open a file && play it...");
								Error("Selected file couldn't be loaded!");
								break;
							}

							// update the position slider
							p = (DWORD)BASS_ChannelBytes2Seconds(chan, BASS_ChannelGetLength(chan));
							MESS(IDC_POS,TBM_SETRANGEMAX,0,p);
							MESS(IDC_POS,TBM_SETPOS,TRUE,0);

							// get the original sample rate
							BASS_ChannelGetAttributes(chan, &freq, NULL, NULL);

							// create a new stream - decoded & resampled :)
							if (!(chan=BASS_FX_TempoCreate(chan, BASS_SAMPLE_LOOP|BASS_FX_FREESOURCE))){
								MESS(ID_OPEN,WM_SETTEXT,0,"click here to open a file && play it...");
								Error("Couldn't create a resampled stream!");
								BASS_StreamFree(chan);
								BASS_MusicFree(chan);
								break;
							}

							// set dsp eq to channel
							SetDSP_EQ(0.0f, 2.5f, 0.0f, 125.0f, 1000.0f, 8000.0f);

							// update the Button to show the loaded file name
							MESS(ID_OPEN,WM_SETTEXT,0,GetFileName(file));

							// set Volume
							p = MESS(IDC_VOL,TBM_GETPOS,0,0);
							BASS_ChannelSetAttributes(chan,-1,p,-101);

							// update tempo sliders
							MESS(IDC_RATE,TBM_SETRANGEMAX,0,(long)((float)freq * maxp));
							MESS(IDC_RATE,TBM_SETRANGEMIN,0,(long)((float)freq * minp));
							MESS(IDC_RATE,TBM_SETPOS,TRUE,freq);
							MESS(IDC_RATE,TBM_SETPAGESIZE,0,(long)((float)freq * 0.01f));		// by 1%

							sprintf(c,"Samplerate = %dHz", freq);
							MESS(IDC_SRATE,WM_SETTEXT,0,c);

							MESS(IDC_TEMPO,TBM_SETPOS,TRUE,0);
							MESS(IDC_STEMPO,WM_SETTEXT,0,"Tempo = 0%");

							MESS(IDC_PITCH,TBM_SETPOS,TRUE,0);
							MESS(IDC_SPITCH,WM_SETTEXT,0,"Pitch Scaling = 0 semitones");

							// update the approximate time in seconds view
							UpdatePositionLabel();

							// play new created stream
							BASS_ChannelPlay(chan,FALSE);
						}
					}
					return 1;
			}
			break;

		case WM_VSCROLL:
			{
				p=SetScroller(l,w,LINE,PAGE,MAX);
				SendMessage((HWND)l,SBM_SETPOS,p,1);
				UpdateFX(GetDlgCtrlID((HWND)l)-ID_C0);
			}
			break;

		case WM_HSCROLL:
			{
				if(!BASS_ChannelIsActive(chan)) break;

				switch (GetDlgCtrlID((HWND)l)) {
					case IDC_TEMPO:
					case IDC_RATE:
					case IDC_PITCH:
						{
							// set new tempo, samplerate & pitch scale
							BASS_FX_TempoSet(chan, (float)MESS(IDC_TEMPO, TBM_GETPOS, 0, 0), (float)MESS(IDC_RATE, TBM_GETPOS, 0, 0), (float)MESS(IDC_PITCH, TBM_GETPOS, 0, 0));

							// update tempo, samplerate & pitch static text
							sprintf(c,"Tempo = %d%%", MESS(IDC_TEMPO, TBM_GETPOS, 0, 0));
							MESS(IDC_STEMPO,WM_SETTEXT,0,c);

							sprintf(c,"Samplerate = %dHz", MESS(IDC_RATE, TBM_GETPOS, 0, 0));
							MESS(IDC_SRATE,WM_SETTEXT,0,c);

							sprintf(c,"Pitch Scaling = %d semitones", MESS(IDC_PITCH, TBM_GETPOS, 0, 0));
							MESS(IDC_SPITCH,WM_SETTEXT,0,c);

							{
								BASS_FX_DSPPEAKEQ eq;

								// Update eq sample rate after changing sample rate
								eq.lBand = 0;	// we must get some band info :)
								BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
									eq.lFreq = MESS(IDC_RATE, TBM_GETPOS, 0, 0);
								BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, &eq);
							}

							// update the approximate time in seconds view
							UpdatePositionLabel();
						}
					break;
					case IDC_POS:
						// change the position
						if (LOWORD(w) == SB_ENDSCROLL) { // seek to new pos
							BASS_ChannelSetPosition(chan, (QWORD)BASS_ChannelSeconds2Bytes(chan, (float)MESS(IDC_POS,TBM_GETPOS,0,0)));
						}
						// update the approximate time in seconds view
						UpdatePositionLabel();
					break;
					case IDC_VOL:
						BASS_ChannelSetAttributes(chan,-1,MESS(IDC_VOL,TBM_GETPOS,0,0),-101);
					break;
				}
			}
			return 1;
			break;

		case WM_CLOSE:
			KillTimer(h,1);
			EndDialog(h, 0);
			return 1;
		break;

		case WM_INITDIALOG:
			win=h;
			GetCurrentDirectory(MAX_PATH,path);
			memset(&ofn,0,sizeof(ofn));
			ofn.lStructSize=sizeof(ofn);
			ofn.hwndOwner=h;
			ofn.hInstance=inst;
			ofn.nMaxFile=MAX_PATH;
			ofn.Flags=OFN_HIDEREADONLY|OFN_EXPLORER;

			// setup output - default device, 44100hz, stereo, 16 bits
			if (!BASS_Init(-1,44100,0,win,NULL)) {
				Error("Can't initialize device");
				DestroyWindow(win);
				return 1;
			}

			// initialize DSP EQ sliders
			MESS(ID_C0,SBM_SETRANGE,0,20);
			MESS(ID_C0,SBM_SETPOS,10,1);

			MESS(ID_C1,SBM_SETRANGE,0,20);
			MESS(ID_C1,SBM_SETPOS,10,1);

			MESS(ID_C2,SBM_SETRANGE,0,20);
			MESS(ID_C2,SBM_SETPOS,10,1);
    
			// volume range
			MESS(IDC_VOL,TBM_SETRANGEMAX,0,100);
			MESS(IDC_VOL,TBM_SETPOS,TRUE,50);
			MESS(IDC_VOL,TBM_SETPAGESIZE,0,5);

			// tempo
			MESS(IDC_TEMPO,TBM_SETRANGEMAX,TRUE,30);
			MESS(IDC_TEMPO,TBM_SETRANGEMIN,TRUE,-30);
			MESS(IDC_TEMPO,TBM_SETPOS,TRUE,0);
			MESS(IDC_TEMPO,TBM_SETPAGESIZE,0,1);

			// rate
			MESS(IDC_RATE,TBM_SETRANGEMAX,0,(long)((float)44100 * maxp));
			MESS(IDC_RATE,TBM_SETRANGEMIN,0,(long)((float)44100 * minp));
			MESS(IDC_RATE,TBM_SETPOS,TRUE,44100);
			MESS(IDC_RATE,TBM_SETPAGESIZE,0,(long)((float)44100 * 0.01f));	// by 1%

			// pitch
			MESS(IDC_PITCH,TBM_SETRANGEMAX,TRUE,30);
			MESS(IDC_PITCH,TBM_SETRANGEMIN,TRUE,-30);
			MESS(IDC_PITCH,TBM_SETPOS,TRUE,0);
			MESS(IDC_PITCH,TBM_SETPAGESIZE,0,1);

			// set bold font for IDC_SDSPEQ
			Font = CreateFont(-11,0,0,0,FW_BOLD,FALSE,FALSE,FALSE,ANSI_CHARSET,OUT_STROKE_PRECIS,CLIP_STROKE_PRECIS,DRAFT_QUALITY,DEFAULT_PITCH|FF_DONTCARE,"MS Sans Serif");
			MESS(IDC_SDSPEQ,WM_SETFONT,Font,0);

			//SetTimer(h,1,100,&Update);

			return 1;
	}
	return 0;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
	inst=hInstance;

	// check that BASS 2.2 was loaded
	if (BASS_GetVersion()!=MAKELONG(2,2)) {
		MessageBox(0,"BASS version 2.2 was not loaded!","Incorrect BASS.DLL",MB_ICONERROR);
		return 1;
	}

	DialogBox(inst,(char*)1000,0,&dialogproc);

	BASS_Free();

	DeleteObject(Font);

	return 0;
}