{========================================================================
 dsp.pas - Copyright (c) 2003-2005 (: JOBnik! :) [Arthur Aminov, ISRAEL]
                                                 [http://www.jobnik.org]
                                                 [   bass_fx@jobnik.org]

 BASS_FX dsp fx in action
 * Requires: BASS.DLL & BASS.PAS 2.2 available @ www.un4seen.com
========================================================================}

unit dsp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, BASS, BASS_FX;

type
  TfrmDSP = class(TForm)
    btnOpen: TButton;
    od: TOpenDialog;
    GroupBox: TGroupBox;
    lbl1KHz: TLabel;
    lbl8KHz: TLabel;
    lblDryMix: TLabel;
    lblWetMix: TLabel;
    lblFeedback: TLabel;
    lblDX: TLabel;
    lbl125Hz: TLabel;
    lblRate: TLabel;
    vsEQBASS: TScrollBar;
    vsEQMID: TScrollBar;
    vsEQTREBLE: TScrollBar;
    chkPhaser: TCheckBox;
    tbDryMix: TTrackBar;
    tbWetMix: TTrackBar;
    tbFeedback: TTrackBar;
    tbDXRate: TTrackBar;
    chkEqualizer: TCheckBox;
    tbRate: TTrackBar;
    lblRange: TLabel;
    tbRange: TTrackBar;
    lblFreq: TLabel;
    tbFreq: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure lblDXClick(Sender: TObject);
    procedure tbDXRateChange(Sender: TObject);
    procedure chkEqualizerClick(Sender: TObject);
    procedure vsEQBASSChange(Sender: TObject);
    procedure vsEQMIDChange(Sender: TObject);
    procedure vsEQTREBLEChange(Sender: TObject);
    procedure chkPhaserClick(Sender: TObject);
    procedure tbDryMixChange(Sender: TObject);
    procedure tbWetMixChange(Sender: TObject);
    procedure tbFeedbackChange(Sender: TObject);
    procedure Error(msg: string);
    procedure UpdateEQ(b, pos: integer);
    procedure UpdatePHS();
    function GetFileName(fp: PChar): PChar;
  private
    { Private declarations }
    chan: DWORD;               // a handle
    freq: DWORD;               // sample rate
    floatable: DWORD;	       // floating-point channel support?
    eq  : BASS_FX_DSPPEAKEQ;   // dsp peaking equalizer
    phs : BASS_FX_DSPPHASER;   // dsp phaser
    vol : DWORD;
    pan : Integer;
  public
    { Public declarations }
  end;

var
  frmDSP: TfrmDSP;

const
  maxp : float = 1.3; // increase dx rate for max 30%
  minp : float = 0.7; // decrease dx rate for min 30%

implementation

{$R *.DFM}

// display error messages
procedure TFrmDSP.Error(msg: string);
var s: string;
begin
    s := msg + #13#10 + #13#10 + 'BASS error Code: ' + IntToStr(BASS_ErrorGetCode);

  MessageBox(handle, PChar(s), 'Error', MB_ICONEXCLAMATION);
end;

procedure TfrmDSP.FormCreate(Sender: TObject);
begin

  // Check that BASS 2.2 is loaded
  If BASS_GetVersion() <> MAKELONG(2,2) then begin
    application.MessageBox('BASS version 2.1 was not loaded!', 'Incorrect BASS.DLL', MB_ICONERROR or MB_OK);
    application.Terminate;
  end;

  // enable floating-point DSP
  BASS_SetConfig(BASS_CONFIG_FLOATDSP, 1);

  // setup output - default device, 44100hz, stereo, 16 bits
  If BASS_Init(-1, 44100, 0, frmDSP.handle, nil) = false then begin
    Error('Couldnt Initialize Digital Output');
    application.Terminate;
  end;

  // check for floating-point capability
  floatable := BASS_StreamCreate(44100, 2, BASS_SAMPLE_FLOAT, nil, 0);
  if (floatable > 0) then begin
    BASS_StreamFree(floatable);  //woohoo!
    floatable := BASS_SAMPLE_FLOAT;
  end;

  freq := 44100;
end;

procedure TfrmDSP.FormDestroy(Sender: TObject);
begin
 BASS_Free;    // free BASS
end;

procedure TfrmDSP.btnOpenClick(Sender: TObject);
begin
  od.Filename := '';
  od.Filter :=  'Playable files|*.mo3;*.xm;*.mod;*.s3m;*.it;*.mtm;*.mp3;*.mp2;*.mp1;*.ogg;*.wav;*.aif|All files|*.*';
  If od.Execute = false then exit;

  // free previous streams/music/dsps handles
  BASS_StreamFree(chan);
  BASS_MusicFree(chan);

  // check for a stream
  chan := BASS_StreamCreateFile(FALSE, pChar(od.filename), 0, 0, BASS_SAMPLE_LOOP Or floatable);

  // check for MOD
  If chan = 0 Then chan := BASS_MusicLoad(FALSE, pChar(od.filename), 0, 0, BASS_MUSIC_LOOP Or BASS_MUSIC_RAMP Or floatable, 0);

  If chan = 0 then begin
    btnOpen.Caption := 'click here to open a file && play it...';
    Error('Selected file couldnt be loaded!');
    Exit;
  end;

  // set dsp effects
  chkEqualizer.OnClick(Self);
  chkPhaser.OnClick(Self);

  // set the dx sample rate
  BASS_ChannelGetAttributes(chan, freq, vol, pan);

  with tbDXRate do begin
    min := 0;    // to avoit min > max
    max := 1;    // to avoid max < min
    max := round(freq * maxp);
    min := round(freq * minp);
    pagesize := round(freq * 0.01);      // by 1%
  end;

  // update the button to show the loaded filename
  btnOpen.Caption := GetFileName(pChar(od.Filename));

  // update lblDX Hint
  lblDX.hint := 'Click to set DX Samplerate to ' + IntToStr(freq) + 'Hz';

  // it must be one of them :)
  BASS_ChannelPlay(chan, FALSE);

  // update the dx sample rate to normal
  lblDX.OnClick(self);
end;

procedure TfrmDSP.lblDXClick(Sender: TObject);
begin
  tbDXRate.Position := freq;
  tbDXRate.OnChange(self);
end;

procedure TfrmDSP.tbDXRateChange(Sender: TObject);
begin
  If BASS_ChannelIsActive(chan) = 0 Then Exit;

  BASS_ChannelSetAttributes(chan, tbDXRate.position, -1, -101);

  lblDX.Caption := 'DirectX Samplerate = ' + IntToStr(tbDXRate.position) + 'Hz';

  // update eq sample rate after changing sample rate
  eq.lBand := 0;        // we must get some band info :)
  BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);
    eq.lFreq := tbDXRate.position;
  BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);
end;

procedure TfrmDSP.chkEqualizerClick(Sender: TObject);
var
  f: DWORD;
begin
  If chkEqualizer.checked Then begin
    // Get current sample rate of a Handle
    BASS_ChannelGetAttributes(chan, f, vol, pan);

    // Set the EQ effect + 1st Band
    BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ, 0);
    BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ, 0); // 2nd Band
    BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PEAKEQ, 0); // 3rd Band ...

    with eq do begin
      lFreq := f;
      fBandwidth := 2.5;
      FQ := 0;
      fGain := 0;

      // set bass
      lBand := 0;
      fCenter := 125;
      BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);

      // set mid
      lBand := 1;
      fCenter := 1000;
      BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);

      // set treble
      lBand := 2;
      fCenter := 8000;
      BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);
    end;

    // update dsp eq
    vsEQBASS.OnChange(Self);
    vsEQMID.OnChange(Self);
    vsEQTREBLE.OnChange(Self);
  end
  Else
    BASS_FX_DSP_Remove(chan, BASS_FX_DSPFX_PEAKEQ);
end;

procedure TfrmDSP.vsEQBASSChange(Sender: TObject);
begin
  UpdateEQ(0, vsEQBASS.Position);
end;

procedure TfrmDSP.vsEQMIDChange(Sender: TObject);
begin
  UpdateEQ(1, vsEQMID.Position);
end;

procedure TfrmDSP.vsEQTREBLEChange(Sender: TObject);
begin
  UpdateEQ(2, vsEQTREBLE.Position);
end;

procedure TfrmDSP.chkPhaserClick(Sender: TObject);
begin
  If chkPhaser.checked Then begin
    BASS_FX_DSP_Set(chan, BASS_FX_DSPFX_PHASER, 1);
      tbDryMix.OnChange(Self);
      tbWetMix.OnChange(Self);
      tbFeedback.OnChange(Self);
      tbRate.OnChange(Self);
      tbRange.OnChange(Self);
      tbFreq.OnChange(Self);
  end
  Else
    BASS_FX_DSP_Remove(chan, BASS_FX_DSPFX_PHASER);
end;

procedure TfrmDSP.tbDryMixChange(Sender: TObject);
begin
  UpdatePHS;
end;

procedure TfrmDSP.tbWetMixChange(Sender: TObject);
begin
  UpdatePHS;
end;

procedure TfrmDSP.tbFeedbackChange(Sender: TObject);
begin
  UpdatePHS;
end;

// update the dsp eq fx
procedure TfrmDSP.UpdateEQ(b, pos: integer);
begin
  eq.lBand := b;    // get b band values

  BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);
    eq.fGain := 10 - pos;
  BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PEAKEQ, @eq);
end;

// update the dsp phaser fx
procedure TfrmDSP.UpdatePHS();
begin
  BASS_FX_DSP_GetParameters(chan, BASS_FX_DSPFX_PHASER, @phs);
    phs.fDryMix := tbDryMix.Position / 1000;
    phs.fWetMix := tbWetMix.Position / 1000;
    phs.fFeedback := tbFeedback.Position / 1000;
    phs.fRate := tbRate.Position / 10;
    phs.fRange := tbRange.Position / 10;
    phs.fFreq := tbFreq.Position / 10;
  BASS_FX_DSP_SetParameters(chan, BASS_FX_DSPFX_PHASER, @phs);
end;

// get the file name from the file path
function TfrmDSP.GetFileName(fp: PChar): PChar;
var P: PChar;
begin
  P := StrRScan(fp, '\');
  If P = nil then begin
    P := StrRScan(fp, ':');
    If P = nil then P := fp;
  end;
  GetFileName := P+1;
end;

end.
